@@ -7,13 +7,14 @@ from fontTools.misc.dictTools import hashdict
 from fontTools.misc.intTools import popCount
 from fontTools.ttLib import newTable
 from fontTools.ttLib.tables import otTables as ot
+from fontTools.ttLib.ttVisitor import TTVisitor
 from fontTools.otlLib.builder import buildLookup, buildSingleSubstSubtable
 from collections import OrderedDict
 
 from .errors import VarLibError, VarLibValidationError
 
 
-def addFeatureVariations(font, conditionalSubstitutions, featureTag='rvrn'):
+def addFeatureVariations(font, conditionalSubstitutions, featureTag="rvrn"):
     """Add conditional substitutions to a Variable Font.
 
     The `conditionalSubstitutions` argument is a list of (Region, Substitutions)
@@ -52,7 +53,9 @@ def addFeatureVariations(font, conditionalSubstitutions, featureTag='rvrn'):
     substitutions = overlayFeatureVariations(conditionalSubstitutions)
 
     # turn substitution dicts into tuples of tuples, so they are hashable
-    conditionalSubstitutions, allSubstitutions = makeSubstitutionsHashable(substitutions)
+    conditionalSubstitutions, allSubstitutions = makeSubstitutionsHashable(
+        substitutions
+    )
     if "GSUB" not in font:
         font["GSUB"] = buildGSUB()
 
@@ -64,24 +67,26 @@ def addFeatureVariations(font, conditionalSubstitutions, featureTag='rvrn'):
     # so rearrange our lookups to match
     conditionsAndLookups = []
     for conditionSet, substitutions in conditionalSubstitutions:
-        conditionsAndLookups.append((conditionSet, [lookupMap[s] for s in substitutions]))
+        conditionsAndLookups.append(
+            (conditionSet, [lookupMap[s] for s in substitutions])
+        )
+
+    addFeatureVariationsRaw(font, font["GSUB"].table, conditionsAndLookups, featureTag)
 
-    addFeatureVariationsRaw(font, font["GSUB"].table,
-                            conditionsAndLookups,
-                            featureTag)
 
 def _checkSubstitutionGlyphsExist(glyphNames, substitutions):
     referencedGlyphNames = set()
     for _, substitution in substitutions:
-        referencedGlyphNames |=  substitution.keys()
-        referencedGlyphNames |=  set(substitution.values())
+        referencedGlyphNames |= substitution.keys()
+        referencedGlyphNames |= set(substitution.values())
     missing = referencedGlyphNames - glyphNames
     if missing:
-       raise VarLibValidationError(
+        raise VarLibValidationError(
             "Missing glyphs are referenced in conditional substitution rules:"
             f" {', '.join(missing)}"
         )
 
+
 def overlayFeatureVariations(conditionalSubstitutions):
     """Compute overlaps between all conditional substitutions.
 
@@ -130,13 +135,13 @@ def overlayFeatureVariations(conditionalSubstitutions):
 
     # Merge same-substitutions rules, as this creates fewer number oflookups.
     merged = OrderedDict()
-    for value,key in conditionalSubstitutions:
+    for value, key in conditionalSubstitutions:
         key = hashdict(key)
         if key in merged:
             merged[key].extend(value)
         else:
             merged[key] = value
-    conditionalSubstitutions = [(v,dict(k)) for k,v in merged.items()]
+    conditionalSubstitutions = [(v, dict(k)) for k, v in merged.items()]
     del merged
 
     # Merge same-region rules, as this is cheaper.
@@ -145,9 +150,13 @@ def overlayFeatureVariations(conditionalSubstitutions):
     # Reversing is such that earlier entries win in case of conflicting substitution
     # rules for the same region.
     merged = OrderedDict()
-    for key,value in reversed(conditionalSubstitutions):
-        key = tuple(sorted((hashdict(cleanupBox(k)) for k in key),
-                           key=lambda d: tuple(sorted(d.items()))))
+    for key, value in reversed(conditionalSubstitutions):
+        key = tuple(
+            sorted(
+                (hashdict(cleanupBox(k)) for k in key),
+                key=lambda d: tuple(sorted(d.items())),
+            )
+        )
         if key in merged:
             merged[key].update(value)
         else:
@@ -158,17 +167,17 @@ def overlayFeatureVariations(conditionalSubstitutions):
     # Overlay
     #
     # Rank is the bit-set of the index of all contributing layers.
-    initMapInit = ((hashdict(),0),) # Initializer representing the entire space
-    boxMap = OrderedDict(initMapInit) # Map from Box to Rank
-    for i,(currRegion,_) in enumerate(conditionalSubstitutions):
+    initMapInit = ((hashdict(), 0),)  # Initializer representing the entire space
+    boxMap = OrderedDict(initMapInit)  # Map from Box to Rank
+    for i, (currRegion, _) in enumerate(conditionalSubstitutions):
         newMap = OrderedDict(initMapInit)
-        currRank = 1<<i
-        for box,rank in boxMap.items():
+        currRank = 1 << i
+        for box, rank in boxMap.items():
             for currBox in currRegion:
                 intersection, remainder = overlayBox(currBox, box)
                 if intersection is not None:
                     intersection = hashdict(intersection)
-                    newMap[intersection] = newMap.get(intersection, 0) | rank|currRank
+                    newMap[intersection] = newMap.get(intersection, 0) | rank | currRank
                 if remainder is not None:
                     remainder = hashdict(remainder)
                     newMap[remainder] = newMap.get(remainder, 0) | rank
@@ -176,19 +185,20 @@ def overlayFeatureVariations(conditionalSubstitutions):
 
     # Generate output
     items = []
-    for box,rank in sorted(boxMap.items(),
-                           key=(lambda BoxAndRank: -popCount(BoxAndRank[1]))):
+    for box, rank in sorted(
+        boxMap.items(), key=(lambda BoxAndRank: -popCount(BoxAndRank[1]))
+    ):
         # Skip any box that doesn't have any substitution.
         if rank == 0:
             continue
         substsList = []
         i = 0
         while rank:
-          if rank & 1:
-              substsList.append(conditionalSubstitutions[i][1])
-          rank >>= 1
-          i += 1
-        items.append((dict(box),substsList))
+            if rank & 1:
+                substsList.append(conditionalSubstitutions[i][1])
+            rank >>= 1
+            i += 1
+        items.append((dict(box), substsList))
     return items
 
 
@@ -201,6 +211,7 @@ def overlayFeatureVariations(conditionalSubstitutions):
 # from the corresponding axes.
 #
 
+
 def overlayBox(top, bot):
     """Overlays ``top`` box on top of ``bot`` box.
 
@@ -222,31 +233,36 @@ def overlayBox(top, bot):
         minimum = max(min1, min2)
         maximum = min(max1, max2)
         if not minimum < maximum:
-            return None, bot # Do not intersect
-        intersection[axisTag] = minimum,maximum
+            return None, bot  # Do not intersect
+        intersection[axisTag] = minimum, maximum
 
     # Remainder
     #
     # Remainder is empty if bot's each axis range lies within that of intersection.
     #
     # Remainder is shrank if bot's each, except for exactly one, axis range lies
-    # within that of intersection, and that one axis, it spills out of the
+    # within that of intersection, and that one axis, it extrudes out of the
     # intersection only on one side.
     #
     # Bot is returned in full as remainder otherwise, as true remainder is not
     # representable as a single box.
 
     remainder = dict(bot)
-    exactlyOne = False
-    fullyInside = False
+    extruding = False
+    fullyInside = True
+    for axisTag in top:
+        if axisTag in bot:
+            continue
+        extruding = True
+        fullyInside = False
+        break
     for axisTag in bot:
-        if axisTag not in intersection:
-            fullyInside = False
-            continue # Axis range lies fully within
+        if axisTag not in top:
+            continue  # Axis range lies fully within
         min1, max1 = intersection[axisTag]
         min2, max2 = bot[axisTag]
         if min1 <= min2 and max2 <= max1:
-            continue # Axis range lies fully within
+            continue  # Axis range lies fully within
 
         # Bot's range doesn't fully lie within that of top's for this axis.
         # We know they intersect, so it cannot lie fully without either; so they
@@ -254,9 +270,9 @@ def overlayBox(top, bot):
 
         # If we have had an overlapping axis before, remainder is not
         # representable as a box, so return full bottom and go home.
-        if exactlyOne:
+        if extruding:
             return intersection, bot
-        exactlyOne = True
+        extruding = True
         fullyInside = False
 
         # Otherwise, cut remainder on this axis and continue.
@@ -272,7 +288,7 @@ def overlayBox(top, bot):
             # Remainder leaks out from both sides.  Can't cut either.
             return intersection, bot
 
-        remainder[axisTag] = minimum,maximum
+        remainder[axisTag] = minimum, maximum
 
     if fullyInside:
         # bot is fully within intersection.  Remainder is empty.
@@ -280,15 +296,16 @@ def overlayBox(top, bot):
 
     return intersection, remainder
 
+
 def cleanupBox(box):
     """Return a sparse copy of `box`, without redundant (default) values.
 
-        >>> cleanupBox({})
-        {}
-        >>> cleanupBox({'wdth': (0.0, 1.0)})
-        {'wdth': (0.0, 1.0)}
-        >>> cleanupBox({'wdth': (-1.0, 1.0)})
-        {}
+    >>> cleanupBox({})
+    {}
+    >>> cleanupBox({'wdth': (0.0, 1.0)})
+    {'wdth': (0.0, 1.0)}
+    >>> cleanupBox({'wdth': (-1.0, 1.0)})
+    {}
 
     """
     return {tag: limit for tag, limit in box.items() if limit != (-1.0, 1.0)}
@@ -298,7 +315,8 @@ def cleanupBox(box):
 # Low level implementation
 #
 
-def addFeatureVariationsRaw(font, table, conditionalSubstitutions, featureTag='rvrn'):
+
+def addFeatureVariationsRaw(font, table, conditionalSubstitutions, featureTag="rvrn"):
     """Low level implementation of addFeatureVariations that directly
     models the possibilities of the FeatureVariations table."""
 
@@ -337,10 +355,13 @@ def addFeatureVariationsRaw(font, table, conditionalSubstitutions, featureTag='r
             langSystems = [lsr.LangSys for lsr in scriptRecord.Script.LangSysRecord]
             for langSys in [scriptRecord.Script.DefaultLangSys] + langSystems:
                 langSys.FeatureIndex.append(varFeatureIndex)
+                langSys.FeatureCount = len(langSys.FeatureIndex)
 
         varFeatureIndices = [varFeatureIndex]
 
-    axisIndices = {axis.axisTag: axisIndex for axisIndex, axis in enumerate(font["fvar"].axes)}
+    axisIndices = {
+        axis.axisTag: axisIndex for axisIndex, axis in enumerate(font["fvar"].axes)
+    }
 
     featureVariationRecords = []
     for conditionSet, lookupIndices in conditionalSubstitutions:
@@ -354,9 +375,17 @@ def addFeatureVariationsRaw(font, table, conditionalSubstitutions, featureTag='r
             conditionTable.append(ct)
         records = []
         for varFeatureIndex in varFeatureIndices:
-            existingLookupIndices = table.FeatureList.FeatureRecord[varFeatureIndex].Feature.LookupListIndex
-            records.append(buildFeatureTableSubstitutionRecord(varFeatureIndex, existingLookupIndices + lookupIndices))
-        featureVariationRecords.append(buildFeatureVariationRecord(conditionTable, records))
+            existingLookupIndices = table.FeatureList.FeatureRecord[
+                varFeatureIndex
+            ].Feature.LookupListIndex
+            records.append(
+                buildFeatureTableSubstitutionRecord(
+                    varFeatureIndex, lookupIndices + existingLookupIndices
+                )
+            )
+        featureVariationRecords.append(
+            buildFeatureVariationRecord(conditionTable, records)
+        )
 
     table.FeatureVariations = buildFeatureVariations(featureVariationRecords)
 
@@ -365,6 +394,7 @@ def addFeatureVariationsRaw(font, table, conditionalSubstitutions, featureTag='r
 # Building GSUB/FeatureVariations internals
 #
 
+
 def buildGSUB():
     """Build a GSUB table from scratch."""
     fontTable = newTable("GSUB")
@@ -379,10 +409,11 @@ def buildGSUB():
     gsub.LookupList.Lookup = []
 
     srec = ot.ScriptRecord()
-    srec.ScriptTag = 'DFLT'
+    srec.ScriptTag = "DFLT"
     srec.Script = ot.Script()
     srec.Script.DefaultLangSys = None
     srec.Script.LangSysRecord = []
+    srec.Script.LangSysCount = 0
 
     langrec = ot.LangSysRecord()
     langrec.LangSys = ot.LangSys()
@@ -413,18 +444,46 @@ def makeSubstitutionsHashable(conditionalSubstitutions):
     return condSubst, sorted(allSubstitutions)
 
 
+class ShifterVisitor(TTVisitor):
+    def __init__(self, shift):
+        self.shift = shift
+
+
+@ShifterVisitor.register_attr(ot.Feature, "LookupListIndex")  # GSUB/GPOS
+def visit(visitor, obj, attr, value):
+    shift = visitor.shift
+    value = [l + shift for l in value]
+    setattr(obj, attr, value)
+
+
+@ShifterVisitor.register_attr(
+    (ot.SubstLookupRecord, ot.PosLookupRecord), "LookupListIndex"
+)
+def visit(visitor, obj, attr, value):
+    setattr(obj, attr, visitor.shift + value)
+
+
 def buildSubstitutionLookups(gsub, allSubstitutions):
     """Build the lookups for the glyph substitutions, return a dict mapping
     the substitution to lookup indices."""
-    firstIndex = len(gsub.LookupList.Lookup)
+
+    # Insert lookups at the beginning of the lookup vector
+    # https://github.com/googlefonts/fontmake/issues/950
+
     lookupMap = {}
     for i, substitutionMap in enumerate(allSubstitutions):
-        lookupMap[substitutionMap] = i + firstIndex
+        lookupMap[substitutionMap] = i
 
-    for subst in allSubstitutions:
+    # Shift all lookup indices in gsub by len(allSubstitutions)
+    shift = len(allSubstitutions)
+    visitor = ShifterVisitor(shift)
+    visitor.visit(gsub.FeatureList.FeatureRecord)
+    visitor.visit(gsub.LookupList.Lookup)
+
+    for i, subst in enumerate(allSubstitutions):
         substMap = dict(subst)
         lookup = buildLookup([buildSingleSubstSubtable(substMap)])
-        gsub.LookupList.Lookup.append(lookup)
+        gsub.LookupList.Lookup.insert(i, lookup)
         assert gsub.LookupList.Lookup[lookupMap[subst]] is lookup
     gsub.LookupList.LookupCount = len(gsub.LookupList.Lookup)
     return lookupMap
@@ -487,10 +546,15 @@ def sortFeatureList(table):
     elsewhere. This is needed after the feature list has been modified.
     """
     # decorate, sort, undecorate, because we need to make an index remapping table
-    tagIndexFea = [(fea.FeatureTag, index, fea) for index, fea in enumerate(table.FeatureList.FeatureRecord)]
+    tagIndexFea = [
+        (fea.FeatureTag, index, fea)
+        for index, fea in enumerate(table.FeatureList.FeatureRecord)
+    ]
     tagIndexFea.sort()
     table.FeatureList.FeatureRecord = [fea for tag, index, fea in tagIndexFea]
-    featureRemap = dict(zip([index for tag, index, fea in tagIndexFea], range(len(tagIndexFea))))
+    featureRemap = dict(
+        zip([index for tag, index, fea in tagIndexFea], range(len(tagIndexFea)))
+    )
 
     # Remap the feature indices
     remapFeatures(table, featureRemap)
@@ -513,11 +577,12 @@ def remapFeatures(table, featureRemap):
 
 
 def _remapLangSys(langSys, featureRemap):
-    if langSys.ReqFeatureIndex != 0xffff:
+    if langSys.ReqFeatureIndex != 0xFFFF:
         langSys.ReqFeatureIndex = featureRemap[langSys.ReqFeatureIndex]
     langSys.FeatureIndex = [featureRemap[index] for index in langSys.FeatureIndex]
 
 
 if __name__ == "__main__":
     import doctest, sys
+
     sys.exit(doctest.testmod().failed)
